#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri Mar 29 15:12:28 2024

@author: gamelina
"""
import numpy as np
import matplotlib.pyplot as plt
from mbtrack2.tracking.element import Element
from scipy.signal import chirp

class Sweep(Element):
    def __init__(self, ring, f0, f1, t1, level):
        self.ring = ring
        self.t = np.arange(0, t1, ring.T0)
        self.N = len(self.t)
        self.count = 0
        self.level = level
        self.sweep = chirp(self.t, f0, t1, f1)

    @Element.parallel
    def track(self, bunch):
        """
        Tracking method for the element.
        No bunch to bunch interaction, so written for Bunch objects and
        @Element.parallel is used to handle Beam objects.
        
        Parameters
        ----------
        bunch : Bunch or Beam object
        """
        sweep_val = self.sweep[self.count]
        bunch["delta"] += self.level / self.ring.E0 * sweep_val
        self.count += 1
        if self.count >= self.N:
            self.count = 0

    def plot(self):
        fig, ax = plt.subplots()
        ax.plot(self.t, self.sweep)
        return fig